@@ -2,7 +2,7 @@
 # extension modules created with SIP.  It provides information about file
 # locations, version numbers etc., and provides some classes and functions.
 #
-# Copyright (c) 2004
+# Copyright (c) 2005
 # 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
 # 
 # This file is part of SIP.
@@ -24,23 +24,24 @@ import types
 _pkg_config = {
     'default_bin_dir':  '/usr/bin',
     'default_mod_dir':  '/usr/lib/python2.4/site-packages',
-    'default_sip_dir':  '/opt/kde/share/sip/',
+    'default_sip_dir':  '/opt/kde/share/sip',
+    'export_all':       0,
     'py_inc_dir':       '/usr/include/python2.4',
     'py_lib_dir':       '/usr/lib/python2.4/config',
-    'py_version':       0x020401,
+    'py_version':       0x020403,
     'qt_dir':           '/usr/lib/qt/',
     'qt_edition':       'free',
     'qt_inc_dir':       '/usr/lib/qt/include',
     'qt_lib':           'qt-mt',
     'qt_lib_dir':       '/usr/lib/qt//lib',
     'qt_threaded':      1,
-    'qt_version':       0x030304,
+    'qt_version':       0x030306,
     'qt_winconfig':     '',
     'sip_bin':          '/opt/kde/bin/sip',
     'sip_inc_dir':      '/usr/include/python2.4',
     'sip_mod_dir':      '/usr/lib/python2.4/site-packages',
-    'sip_version':      0x040101,
-    'sip_version_str':  '4.1.1'
+    'sip_version':      0x040201,
+    'sip_version_str':  '4.2.1'
 }
 
 _default_macros = {
@@ -88,6 +89,7 @@ _default_macros = {
     'CXXFLAGS_WARN_ON':         '-Wall -W',
     'DEFINES':                  '',
     'DEL_FILE':                 'rm -f',
+    'EXTENSION_PLUGIN':         '',
     'EXTENSION_SHLIB':          '',
     'INCDIR':                   '',
     'INCDIR_OPENGL':            '/usr/X11R6/include',
@@ -125,7 +127,8 @@ _default_macros = {
     'MKDIR':                    'mkdir -p',
     'MOC':                      '/usr/lib/qt//bin/moc',
     'RANLIB':                   '',
-    'RPATH':                    '-Wl,-rpath,'
+    'RPATH':                    '-Wl,-rpath,',
+    'STRIP':                    'strip'
 }
 
 # The stack of configuration dictionaries.
@@ -327,7 +330,7 @@ class Makefile:
             # These require special handling as they are (potentially) a set of
             # space separated values rather than a single value that might
             # contain spaces.
-            if m in ("DEFINES", "INCDIR", "LIBDIR"):
+            if m == "DEFINES" or m[:6] in ("INCDIR", "LIBDIR"):
                 val = string.split(val)
 
             # We also want to treat lists of libraries in the same way so that
@@ -416,7 +419,7 @@ class Makefile:
 
         # Don't use a unique list as libraries may need to be searched more
         # than once.  Also MacOS/X uses the form "-framework lib" so we don't
-        # want to lose the mulitple "-framework".
+        # want to lose the multiple "-framework".
         libs = []
 
         for l in self.extra_libs:
@@ -453,6 +456,12 @@ class Makefile:
                     if os.access(bpy_lib_path, os.F_OK):
                         py_lib = bpy_lib
 
+                if self._debug:
+                    py_lib = py_lib + "_d"
+
+                    cflags.append("/D_DEBUG")
+                    cxxflags.append("/D_DEBUG")
+
                 libs.append(self.platform_lib(py_lib))
 
         if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
@@ -700,40 +709,45 @@ class Makefile:
         """
         Parse a build file and return the corresponding dictionary.
 
-        filename is the name of the build file.
+        filename is the name of the build file.  If it is a dictionary instead
+        then its contents are validated.
         """
-        if self._dir:
-            bfname = os.path.join(self._dir, filename)
+        if type(filename) is types.DictType:
+            bfname = "dictionary"
+            dict = filename
         else:
-            bfname = filename
+            if self._dir:
+                bfname = os.path.join(self._dir, filename)
+            else:
+                bfname = filename
 
-        dict = {}
+            dict = {}
 
-        try:
-            f = open(bfname, "r")
-        except IOError, detail:
-            error("Unable to open \"%s\": %s" % (bfname, detail))
+            try:
+                f = open(bfname, "r")
+            except IOError, detail:
+                error("Unable to open \"%s\": %s" % (bfname, detail))
 
-        line_nr = 1
-        line = f.readline()
-
-        while line:
-            line = string.strip(line)
-
-            if line and line[0] != "#":
-                eq = string.find(line, "=")
-
-                if eq <= 0:
-                    error("\"%s\" line %d: Line must be in the form 'name = value value...'." % (bfname, line_nr))
-
-                dict[string.strip(line[:eq])] = string.strip(line[eq + 1:])
-
-            line_nr = line_nr + 1
+            line_nr = 1
             line = f.readline()
 
-        f.close()
+            while line:
+                line = string.strip(line)
 
-        # Get the compulsory values.
+                if line and line[0] != "#":
+                    eq = string.find(line, "=")
+
+                    if eq <= 0:
+                        error("\"%s\" line %d: Line must be in the form 'name = value value...'." % (bfname, line_nr))
+
+                    dict[string.strip(line[:eq])] = string.strip(line[eq + 1:])
+
+                line_nr = line_nr + 1
+                line = f.readline()
+
+            f.close()
+
+        # Check the compulsory values.
         for i in ("target", "sources"):
             try:
                 dict[i]
@@ -754,6 +768,7 @@ class Makefile:
             ext = ".o"
 
         olist = []
+
         for f in string.split(dict["sources"]):
             root, discard = os.path.splitext(f)
             olist.append(root + ext)
@@ -764,6 +779,7 @@ class Makefile:
 
             root, discard = os.path.splitext(f)
             olist.append("moc_" + root + ext)
+
         dict["objects"] = string.join(olist)
 
         headers = []
@@ -808,7 +824,7 @@ class Makefile:
         else:
             mfname = os.path.join(self._builddir, self._makefile)
             srcdir = self._srcdir
-            
+
         try:
             mfile = open(mfname, "w")
         except IOError, detail:
@@ -986,13 +1002,14 @@ class Makefile:
         """
         mfile.write("\ndistclean: clean\n\t-rm *.pyc\n\t-rm Makefile\n")
 
-    def install_file(self, mfile, src, dst):
+    def install_file(self, mfile, src, dst, strip=0):
         """Install one or more files in a directory.
 
         mfile is the file object.
         src is the name of a single file to install, or the list of a number of
         files to install.
         dst is the name of the destination directory.
+        strip is set if the files should be stripped after been installed.
         """
         # Help package builders.
         if self.generator == "UNIX":
@@ -1008,9 +1025,19 @@ class Makefile:
         if type(src) != types.ListType:
             src = [src]
 
-        for sf in src:
-            mfile.write("\t%s %s %s\n" % (self.copy, _quote(sf), _quote(os.path.join(dst, os.path.basename(sf)))))
+        # Get the strip command if needed.
+        if strip:
+            strip_cmd = self.optional_string("STRIP")
 
+            if not strip_cmd:
+                strip = 0
+
+        for sf in src:
+            target = _quote(os.path.join(dst, os.path.basename(sf)))
+            mfile.write("\t%s %s %s\n" % (self.copy, _quote(sf), target))
+            if strip:
+                mfile.write("\t%s %s\n" % (strip_cmd, target))
+                
     def uninstall_file(self, mfile, src, dst):
         """Uninstall one or more files in a directory.
 
@@ -1140,13 +1167,16 @@ class ModuleMakefile(Makefile):
     """
     def __init__(self, configuration, build_file, install_dir=None, static=0,
                  console=0, qt=0, opengl=0, threaded=0, warnings=None, debug=0,
-                 dir=None, makefile="Makefile", installs=None, srcdir=None,
+                 dir=None, makefile="Makefile", installs=None,strip=1, srcdir=None,
                  builddir=None):
         """Initialise an instance of a module Makefile.
 
-        build_file is the file containing the target specific information.
+        build_file is the file containing the target specific information.  If
+        it is a dictionary instead then its contents are validated.
         install_dir is the directory the target will be installed in.
         static is set if the module should be built as a static library.
+        strip is set if the module should be stripped of unneeded symbols when
+        installed.
         """
         Makefile.__init__(self, configuration, console, qt, opengl, 1, threaded, warnings, debug, dir, makefile, installs, srcdir, builddir)
 
@@ -1155,6 +1185,12 @@ class ModuleMakefile(Makefile):
         self._dir = dir
         self.static = static
 
+        # Don't strip if this is a debug or static build.
+        if debug or static:
+            self._strip = 0
+        else:
+            self._strip = strip
+
         # Save the target name for later.
         self._target = self._build["target"]
 
@@ -1166,8 +1202,19 @@ class ModuleMakefile(Makefile):
                 if sys.platform != "win32":
                     self._target = self._target + "module"
 
+        if (sys.platform != "win32" and static) or self.config.sip_version < 0x040000:
             self._target = "lib" + self._target
 
+        if sys.platform == "win32" and debug:
+            self._target = self._target + "_d"
+
+        # See if we should only export the module initialisation function (to
+        # reduce the size of the module and speed up module load times).
+        if self.config.export_all or self.config.sip_version < 0x040000 or debug or static:
+            self._export_init = 0
+        else:
+            self._export_init = 1
+
     def finalise(self):
         """Finalise the macros common to all module Makefiles.
         """
@@ -1176,6 +1223,8 @@ class ModuleMakefile(Makefile):
                 lflags_console = "LFLAGS_CONSOLE"
             else:
                 lflags_console = "LFLAGS_WINDOWS"
+
+            self.DEFINES.append("SIP_STATIC_MODULE")
         else:
             self.CFLAGS.extend(self.optional_list("CFLAGS_SHLIB"))
             self.CXXFLAGS.extend(self.optional_list("CXXFLAGS_SHLIB"))
@@ -1212,17 +1261,38 @@ class ModuleMakefile(Makefile):
                 self.LFLAGS.append("-Wl,--out-implib,%s.a" % self._target)
             elif self.optional_string("AIX_SHLIB"):
                 # AIX needs a lot of special handling.
-                if self.optional_string("LINK_SHLIB_CMD"):
+                if self.required_string('LINK') == 'g++':
+                    # g++ is used for linking.
+                    if self.config.sip_version >= 0x040000:
+                        # For SIP v4 and g++:
+                        # 1.) Import the python symbols
+                        aix_lflags = ['-Wl,-bI:%s/python.exp' % self.config.py_lib_dir]
+
+                        if self._export_init:
+                            aix_lflags.append('-Wl,-bnoexpall')
+                            aix_lflags.append('-Wl,-bnoentry')
+                            aix_lflags.append('-Wl,-bE:%s.exp' % self._target)
+                    else:
+                        # For SIP v3 and g++:
+                        # 1.) Enable runtime linking
+                        # 2.) Import the python symbols
+                        aix_lflags = ['-Wl,-G',
+                                      '-Wl,-bI:%s/python.exp' % self.config.py_lib_dir]
+                else:
+                    # IBM VisualAge C++ is used for linking.
                     if self.config.sip_version >= 0x040000:
                         # For SIP v4 and xlC:
                         # 1.) Create a shared object
-                        # 2.) Don't export all symbols
-                        # 3.) Export the module init function
-                        # 4.) Import the python symbols
+                        # 2.) Import the python symbols
                         aix_lflags = ['-qmkshrobj',
                                       '-bnoexpall',
                                       '-e init%s' % self._build["target"],
                                       '-bI:%s/python.exp' % self.config.py_lib_dir]
+
+                        if self._export_init:
+                            aix_lflags.append('-bnoexpall')
+                            aix_lflags.append('-bnoentry')
+                            aix_lflags.append('-bE:%s.exp' % self._target)
                     else:
                         # For SIP v3 and xlC:
                         # 1.) Enable runtime linking
@@ -1231,24 +1301,18 @@ class ModuleMakefile(Makefile):
                         aix_lflags = ['-G',
                                       '-qmkshrobj',
                                       '-bI:%s/python.exp' % self.config.py_lib_dir]
-                else:
-                    if self.config.sip_version >= 0x040000:
-                        # For SIP v4 and g++:
-                        # 1.) Don't export all symbols
-                        # 2.) Export the module init function
-                        # 3.) Import the python symbols
-                        aix_lflags = ['-Wl,-bnoexpall',
-                                      '-Wl,"-e init%s"' % self._build["target"],
-                                      '-Wl,-bI:%s/python.exp' % self.config.py_lib_dir]
-                    else:
-                        # For SIP v3 and g++:
-                        # 1.) Enable runtime linking
-                        # 2.) Import the python symbols
-                        aix_lflags = ['-Wl,-G',
-                                      '-Wl,-bI:%s/python.exp' % self.config.py_lib_dir]
 
                 self.LFLAGS.extend(aix_lflags)
             else:
+                if self._export_init:
+                    if sys.platform[:5] == 'linux':
+                        self.LFLAGS.extend(['-Wl,--version-script=%s.exp' % self._target])
+                    elif sys.platform[:5] == 'hp-ux':
+                        self.LFLAGS.extend(['-Wl,+e,init%s' % self._target])
+                    elif sys.platform[:5] == 'irix' and self.required_string('LINK') != 'g++':
+                        # Doesn't work when g++ is used for linking on IRIX.
+                        self.LFLAGS.extend(['-Wl,-exported_symbol,init%s' % self._target])
+
                 # Force the shared linker if there is one.
                 link_shlib = self.optional_list("LINK_SHLIB")
 
@@ -1292,7 +1356,9 @@ class ModuleMakefile(Makefile):
             elif sys.platform == "darwin":
                 ext = "so"
             else:
-                ext = self.optional_string("EXTENSION_SHLIB", "so")
+                ext = self.optional_string("EXTENSION_PLUGIN")
+                if not ext:
+                    ext = self.optional_string("EXTENSION_SHLIB", "so")
 
         mfile.write("TARGET = %s\n" % (self._target + "." + ext))
         mfile.write("OFILES = %s\n" % self._build["objects"])
@@ -1366,6 +1432,14 @@ class ModuleMakefile(Makefile):
                 if self._ranlib:
                     mfile.write("\t$(RANLIB) $(TARGET)\n")
             else:
+                if self._export_init:
+                    # Create an export file for AIX and Linux.
+                    if sys.platform[:5] == 'linux':
+                        mfile.write("\t@echo '{ global: init%s; local: *; };' > %s.exp\n" % (self._target, self._target))
+                    elif sys.platform[:3] == 'aix':
+                        mfile.write("\t@echo '#!' >%s.exp" % self._target)
+                        mfile.write("; \\\n\t echo 'init%s' >>%s.exp\n" % (self._target, self._target))
+
                 mfile.write("\t$(LINK) $(LFLAGS) -o $(TARGET) $(OFILES) $(LIBS)\n")
 
         mfile.write("\n$(OFILES): $(HFILES)\n")
@@ -1386,7 +1460,7 @@ class ModuleMakefile(Makefile):
             self._install_dir = self.config.default_mod_dir
 
         mfile.write("\ninstall: $(TARGET)\n")
-        self.install_file(mfile, "$(TARGET)", self._install_dir)
+        self.install_file(mfile, "$(TARGET)", self._install_dir, self._strip)
 
         if self.config.sip_version < 0x040000 and not self.static:
             if self.generator == "MINGW":
@@ -1423,7 +1497,11 @@ class ModuleMakefile(Makefile):
         """
         mfile.write("\nclean:\n")
         self.clean_build_file_objects(mfile, self._build)
-
+        
+        # Remove any export file on AIX and Linux.
+        if self._export_init and (sys.platform[:5] == 'linux' or sys.platform[:3] == 'aix'):
+            mfile.write("\t-%s %s.exp\n" % (self.rm, self._target))
+         
     def generate_target_distclean(self, mfile):
         """Generate the distclean target.
 
@@ -1460,7 +1538,8 @@ class ProgramMakefile(Makefile):
                  installs=None, srcdir=None, builddir=None):
         """Initialise an instance of a program Makefile.
 
-        build_file is the file containing the target specific information.
+        build_file is the file containing the target specific information.  If
+        it is a dictionary instead then its contents are validated.
         install_dir is the directory the target will be installed in.
         """
         Makefile.__init__(self, configuration, console, qt, opengl, python, threaded, warnings, debug, dir, makefile, installs, srcdir, builddir)
@@ -1486,7 +1565,7 @@ class ProgramMakefile(Makefile):
         if sys.platform == "win32":
             exe = exe + ".exe"
         exe = os.path.join(self._srcdir,exe)
-        
+
         # The command line.
         build = []
 
